"""Metrics entities module"""

import os
import tempfile
from typing import Literal, Protocol

# pylint: disable=R0801
if not os.environ.get("PROMETHEUS_MULTIPROC_DIR"):
    TEMPDIR = tempfile.TemporaryDirectory()  # pylint: disable=R1732
    MULTIPROC_DIR = TEMPDIR.name
    os.environ["PROMETHEUS_MULTIPROC_DIR"] = TEMPDIR.name
if not os.environ.get("PROMETHEUS_DISABLE_CREATED_SERIES"):
    os.environ["PROMETHEUS_DISABLE_CREATED_SERIES"] = "true"
# pylint: disable=C0413
from prometheus_client import CollectorRegistry, Counter

from ..errors.errors import ErrorInfo


class BaseMetricsContainer:
    """
    Base class for Metrics manager properties
    Attributes:
        requestLabels: Label names for the request metrics.
        errorLabels: Label names for the error metrics.
        errorCount: Counter for error metrics.
        requestCount: Counter for request metrics.
        extraValues: Label values for both metrics.
        multiprocEnabled: Whether or not to enable multiprocessing mode.
    """

    __slots__ = (
        "requestLabels",
        "errorLabels",
        "errorCount",
        "requestCount",
        "extraValues",
        "multiprocEnabled",
    )

    def __init__(self, extraLabels: dict[str, str] | None = None):
        extraLabels = extraLabels or {}
        self.errorLabels = ["error_code", "path", *extraLabels.keys()]
        self.requestLabels = ["status_code", "path", *extraLabels.keys()]
        self.extraValues = [*extraLabels.values()]
        self.errorCount = None
        self.requestCount = None
        self.multiprocEnabled = True

    def initialize(self, registry: CollectorRegistry):
        """
        Initialize the counters.
        Args:
            registry: Prometheus collector registry
        """
        extraArgs = {} if self.multiprocEnabled else {"registry": registry}

        self.errorCount = Counter("request_errors", "Counter of request errors", self.errorLabels, **extraArgs)
        self.requestCount = Counter("request_count", "Counter of requests", self.requestLabels, **extraArgs)

    def handleRequest(self, request, response):
        """Add point data to metrics"""
        path = f"{request.method}:{request.beautyRoute}"
        self.requestCount.labels(response.status, path, *self.extraValues).inc()
        if isinstance(request.error, ErrorInfo):
            self.errorCount.labels(request.error.errorCode, path, *self.extraValues).inc()


class MetricsSettings(Protocol):
    """Metrics settings protocol"""

    # whether metrics are enabled
    enabled: Literal[0, 1]
    # metrics output format
    metricsFormat: Literal["prometheus"]
    # whether multiprocessing is enabled
    multiprocEnabled: Literal[0, 1]
